"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_extra_1 = require("fs-extra");
const xml_js_1 = require("xml-js");
const path = require('path');
class LaravelTestsFactory {
    getName() {
        return 'factory-tests-laravel';
    }
    execute(model, config) {
        return __awaiter(this, void 0, void 0, function* () {
            const warnings = [];
            const outputDir = config.output;
            yield fs_extra_1.ensureDir(path.resolve(__dirname, config.output));
            try {
                yield LaravelTestsFactory.updatePhpUnitConfig(outputDir);
            }
            catch (error) {
                if (error.code === 'ENOENT') {
                    warnings.push('No PHPUnit configuration file found. Could not add TestSuite entry.');
                }
                else {
                    throw error;
                }
            }
            try {
                yield LaravelTestsFactory.updateComposerConfig();
            }
            catch (error) {
                if (error.code === 'ENOENT') {
                    warnings.push('No composer.json file found. Could not add PHP dependencies.');
                }
                else {
                    throw error;
                }
            }
            yield LaravelTestsFactory.writeJsonSchemaFiles(model, config);
            yield LaravelTestsFactory.createHookTraitFile(config);
            yield LaravelTestsFactory.createTestCases(model.decorated.testSuite.testCases, config);
            return Promise.resolve(warnings);
        });
    }
    static updatePhpUnitConfig(outputDir) {
        return __awaiter(this, void 0, void 0, function* () {
            const file = yield fs_extra_1.readFile('phpunit.xml', 'utf-8');
            const object = yield xml_js_1.xml2js(file, { compact: true });
            const testSuite = {
                _attributes: {
                    name: 'Comet',
                },
                directory: {
                    _attributes: {
                        suffix: 'Test.php',
                    },
                    _text: `./${outputDir}`,
                },
            };
            if (object.phpunit.testsuites && Array.isArray(object.phpunit.testsuites.testsuite)) {
                const testSuites = object.phpunit.testsuites.testsuite;
                const index = testSuites.findIndex(entry => entry._attributes.name === 'Comet');
                if (index !== -1) {
                    testSuites.splice(index, 1);
                }
                testSuites.push(testSuite);
                object.phpunit.testsuites.testsuite = testSuites;
            }
            else {
                object.phpunit.testsuites = {
                    testsuite: [testSuite],
                };
            }
            const xml = xml_js_1.js2xml(object, { compact: true, ignoreComment: true, spaces: 4 });
            yield fs_extra_1.writeFile('phpunit.xml', xml, 'utf8');
        });
    }
    static updateComposerConfig() {
        return __awaiter(this, void 0, void 0, function* () {
            const file = yield fs_extra_1.readFile('composer.json', 'utf-8');
            const object = JSON.parse(file);
            object['require-dev']['estahn/phpunit-json-assertions'] = '^3.0';
            yield fs_extra_1.writeJson('composer.json', object, { spaces: 4 });
        });
    }
    static writeJsonSchemaFiles(model, config) {
        return __awaiter(this, void 0, void 0, function* () {
            const usedSchemas = new Set(model.decorated.testSuite.testCases.map((testCase) => {
                return testCase.schema;
            }));
            const schemasToWrite = model.decorated.jsonSchemas.filter((action) => {
                return action.$operation === 'response' && usedSchemas.has(action.$name) === true;
            });
            for (let i = 0; i < schemasToWrite.length; i += 1) {
                const action = schemasToWrite[i];
                yield fs_extra_1.ensureDir(path.join(config.output, 'schemas'));
                const file = path.join(config.output, 'schemas', `${action.$name}.json`);
                yield fs_extra_1.writeJson(file, action.schema, { spaces: 4 });
            }
        });
    }
    static createHookTraitFile(config) {
        return __awaiter(this, void 0, void 0, function* () {
            yield fs_extra_1.writeFile(path.join(config.output, 'HasHooks.php'), `<?php

namespace Tests\\Comet;

trait HasHooks
{
    /**
     * Hook to be executed before every request.
     * You can customize the request here, e.g.:
     *      return $this->withHeaders([
     *          'X-Authorization' => abcddefg12345,
     *      ]);
     * @return $this
     */
    protected function before() {
        return $this;
    }

}

`);
        });
    }
    static createTestCases(testCases, config) {
        return __awaiter(this, void 0, void 0, function* () {
            let body = LaravelTestsFactory.getFileHeader();
            const baseUrl = config.base_url;
            const outputDir = config.output;
            testCases.forEach((testCase) => {
                const url = LaravelTestsFactory.getResolvedUrl(baseUrl, testCase);
                if (testCase.isFaulty === false) {
                    body += LaravelTestsFactory.createNominalDefinition(testCase, url, outputDir);
                }
                else {
                    body += LaravelTestsFactory.createFaultyDefinition(testCase, url);
                }
            });
            body += LaravelTestsFactory.getFileFooter();
            yield fs_extra_1.writeFile(path.join(config.output, 'CometApiTest.php'), body);
        });
    }
    static getResolvedUrl(baseUrl, testCase) {
        let url = testCase.path;
        let hasAddedQueryParameter = false;
        testCase.parameters.forEach((parameter) => {
            if (parameter.location === 'path') {
                const replace = `{${parameter.name}}`;
                const expression = new RegExp(replace, 'gi');
                url = url.replace(expression, parameter.value);
            }
            if (parameter.location === 'query') {
                const separator = hasAddedQueryParameter ? '&' : '?';
                url = `${url}${separator}${parameter.name}=${parameter.value}`;
                hasAddedQueryParameter = true;
            }
        });
        return `${baseUrl}${url}`;
    }
    static getFileHeader() {
        return `<?php

namespace Tests\\Comet;

use Tests\\TestCase;
use Illuminate\\Foundation\\Testing\\RefreshDatabase;
use EnricoStahn\\JsonAssert\\Assert as JsonAssert;

class CometApiTest extends TestCase
{
    use RefreshDatabase, JsonAssert, HasHooks;`;
    }
    static createNominalDefinition(testCase, url, outputDir) {
        return `

    public function test${testCase.name.charAt(0).toUpperCase() + testCase.name.slice(1)}()
    {
        $body = ${testCase.hasRequestBody ? `'${testCase.requestBody}'` : 'null'};
        $headers = $this->getJsonHeaders($body);
        $response = $this->executeRequest('${testCase.method.toUpperCase()}', '${url}', $headers, $body);
        $response->assertSuccessful();
        ${testCase.schema ? this.getJsonSchemaAssertion(outputDir, testCase) : ''}
    }`;
    }
    static createFaultyDefinition(testCase, url) {
        return `

    public function test${testCase.name.charAt(0).toUpperCase() + testCase.name.slice(1)}()
    {
        $body = ${testCase.hasRequestBody ? `'${testCase.requestBody}'` : '\'\''};
        $headers = $this->getJsonHeaders($body);
        $response = $this->executeRequest('${testCase.method.toUpperCase()}', '${url}', $headers, $body);
        $this->assertHasClientError($response);
    }`;
    }
    static getFileFooter() {
        return `

    /**
     * Assert that a test response returned a client error (4xx).
     * @param \\Illuminate\\Foundation\\Testing\\TestResponse $response
     */
    protected function assertHasClientError($response)
    {
        $this->assertTrue(
            $response->isClientError(),
            'Response status code ['.$response->getStatusCode().'] is not a client error status code.'
        );
    }

    /**
     * Get default JSON request headers.
     * @param string $body
     * @return array
     */
    protected function getJsonHeaders($body)
    {
        return  [
            'CONTENT_LENGTH' => mb_strlen($body, '8bit'),
            'CONTENT_TYPE' => 'application/json',
            'Accept' => 'application/json',
        ];
    }

    /**
     * Execute the test request.
     * @param $method
     * @param $path
     * @param $headers
     * @param $body
     * @return \\Illuminate\\Foundation\\Testing\\TestResponse
     */
    protected function executeRequest($method, $path, $headers, $body)
    {
        return $this
            ->before()
            ->call($method, $path, [], [], [], $this->transformHeadersToServerVars($headers), $body);
    }
}
    `;
    }
    static getJsonSchemaAssertion(outputDir, testCase) {
        const schemaPath = `base_path().'/${outputDir}/schemas/${testCase.schema}.json'`;
        return `$responseContent = $response->getContent();
        $this->assertJsonMatchesSchema(json_decode($responseContent), ${schemaPath});`;
    }
}
exports.default = LaravelTestsFactory;
//# sourceMappingURL=index.js.map